﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Web;
using System.Web.Services;
using System.Drawing;
using System.IO;
using System.Drawing.Drawing2D;

namespace GuptaSoap
{
    /// <summary>
    /// Zusammenfassungsbeschreibung für GuptaSoap
    /// </summary>
    [WebService(Namespace = "http://guptacom.org/")]
    [WebServiceBinding(ConformsTo = WsiProfiles.BasicProfile1_1)]
    [System.ComponentModel.ToolboxItem(false)]
    // Um das Aufrufen dieses Webdiensts aus einem Skript mit ASP.NET AJAX zuzulassen, heben Sie die Auskommentierung der folgenden Zeile auf. 
    // [System.Web.Script.Services.ScriptService]
    public class GuptaSoap : System.Web.Services.WebService
    {

        public struct ContactSaveResult
        {
            public Boolean bRet;
            public String sErrorMsg;
        }

        [WebMethod]
        public ContactSaveResult ContactSave(String sFirstName, String sLastName, DateTime dtBirthday, Int32 nLuckyNumber)
        {
            ContactSaveResult cResult = new ContactSaveResult();
            try
            {
                GuptaCom.MyCom cGc = new GuptaCom.MyCom();
                cResult.bRet = cGc.ContactSave(sFirstName, sLastName, dtBirthday, nLuckyNumber, ref cResult.sErrorMsg);
            }
            catch (Exception e)
            {
                cResult.bRet = false;
                cResult.sErrorMsg = e.Message;
            }
            return cResult;
        }

        public struct ContactSelectResult
        {
            public Boolean bRet;
            public String sErrorMsg;
            public Int32 nContactId;
            public String sFirstName;
            public String sLastName;
            public DateTime dtBirthday;
            public Int32 nLuckyNumber;
            public String sImagePath;
        }

        [WebMethod]
        public ContactSelectResult ContactSelect(Int32 nContactId )
        {
            ContactSelectResult cResult = new ContactSelectResult();
            Object oImage = null;
            Image img = null;
            String sFilename = "";
            String sMyUri = "";
            try
            {
                GuptaCom.MyCom cGc = new GuptaCom.MyCom();
                cResult.bRet = cGc.ContactSelect(nContactId, ref cResult.sFirstName, ref cResult.sLastName, ref cResult.dtBirthday, ref cResult.nLuckyNumber, ref oImage, ref cResult.sErrorMsg);
                if (cResult.bRet)
                {
                    cResult.nContactId = nContactId; // ja, ja, das ist redundant. Aber nützlich.
                    try // Da wir nicht sicher sein können, dass es sich bei dem Byte-Array wirklich um Bilddaten handelt, könnte im Folgenden eine Exception ausgelöst werden
                    {
                        // Das allgemein Objekt oImage wird in ein Byte-Array gecastet (was es ja auch ist)
                        // mit diesem wird ein Memory-Stream initialisiert
                        // das wiederum dem Image zugewiesen wird.
                        img = Image.FromStream(new MemoryStream((byte[])oImage));
                        // Das Image wird in Form eines Jpeg in den Unterordner Images abgelegt
                        sFilename = this.Server.MapPath("./images/" + nContactId.ToString() + ".jpg");
                        img.Save(sFilename, System.Drawing.Imaging.ImageFormat.Jpeg);
                        // und über folgende URL ist das Bild abbrufbar
                        cResult.sImagePath = "/images/" + nContactId.ToString() + ".jpg";

                        //aus der relativen URL eine absolute erstellen
                        sMyUri = this.Context.Request.Url.AbsoluteUri;
                        sMyUri = sMyUri.Substring(0, sMyUri.LastIndexOf(Context.Request.Url.LocalPath));
                        cResult.sImagePath = sMyUri + cResult.sImagePath;
                    }
                    catch (Exception ex)
                    {
                        // Bild konnte nicht umgewandelt werden (z.B. weil in der DB leer, oder unverständliches Format
                        // interessiert uns nicht weiter; Image-Pfad wird nicht rausgegeben
                        cResult.sErrorMsg = "Bad Imagedata: " + ex.Message;
                    }
                }
            }
            catch (Exception ex)
            {
                cResult.bRet = false;
                cResult.sErrorMsg = ex.Message;
            }
            return cResult;
        }

        public struct ContactUpdateResult
        {
            public Boolean bRet;
            public String sErrorMsg;
        }

        [WebMethod]
        public ContactUpdateResult ContactUpdate(Int32 nContactId, String sFirstName, String sLastName, DateTime dtBirthday, Int32 nLuckyNumber, String sImagePath)
        {
            ContactUpdateResult cResult = new ContactUpdateResult();
            Object oImage = null;

            try
            {
                GuptaCom.MyCom cGc = new GuptaCom.MyCom();
                if (sImagePath.Length > 0)
                {
                    if (sImagePath.ToLower() == "remove")
                        oImage = new byte[7] {114, 101, 109, 111, 118, 101, 0}; //ASCII for "remove" - wird in der COM-Funktion entsprechend ausgewertet
                    else
                    {
                        try
                        {
                            Image imgRaw = Image.FromFile(sImagePath);
                            Image imgDest = SizeToBestFit(imgRaw);
                            MemoryStream streamImage = new MemoryStream();
                            imgDest.Save(streamImage, System.Drawing.Imaging.ImageFormat.Jpeg);
                            imgRaw.Dispose();
                            oImage = streamImage.GetBuffer();
                        }
                        catch
                        {
                            cResult.sErrorMsg = "Could not find image";
                            //trotzdem weitermachen - nicht false zurückgeben
                        }
                    }
                }
                cResult.bRet = cGc.ContactUpdate(nContactId, sFirstName, sLastName, dtBirthday, nLuckyNumber, oImage, ref cResult.sErrorMsg);

            }
            catch (Exception ex)
            {
                cResult.bRet = false;
                cResult.sErrorMsg = ex.Message;
            }
            if (cResult.bRet)
            {
                try
                {
                    System.IO.File.Delete(sImagePath); // clean up - image is in DB now (hopefully)
                }
                catch
                {
                    // do nothing
                }
            }
            return cResult;
        }

        public struct ContactInsertResult
        {
            public Boolean bRet;
            public Int32 nContactId;
            public String sErrorMsg;
        }

        [WebMethod]
        public ContactInsertResult ContactInsert(String sFirstName, String sLastName, DateTime dtBirthday, Int32 nLuckyNumber, String sImagePath)
        {
            ContactInsertResult cResult = new ContactInsertResult();
            Object oImage = null;

            try
            {
                GuptaCom.MyCom cGc = new GuptaCom.MyCom();
                if (sImagePath.Length > 0)
                {
                    Image imgRaw = Image.FromFile(sImagePath);
                    Image imgDest = SizeToBestFit(imgRaw);
                    MemoryStream streamImage = new MemoryStream();
                    imgDest.Save(streamImage, System.Drawing.Imaging.ImageFormat.Jpeg);
                    imgRaw.Dispose();
                    oImage = streamImage.GetBuffer();
                }
                cResult.bRet = cGc.ContactInsert(ref cResult.nContactId, sFirstName, sLastName, dtBirthday, nLuckyNumber, oImage, ref cResult.sErrorMsg);
            }
            catch (Exception ex)
            {
                cResult.bRet = false;
                cResult.sErrorMsg = ex.Message;
            }
            if (cResult.bRet)
            {
                try
                {
                    System.IO.File.Delete(sImagePath); // clean up - image is in DB now (hopefully)
                }
                catch
                {
                    // do nothing
                }
            }
            return cResult;
        }

        public struct ContactDeleteResult
        {
            public Boolean bRet;
            public String sErrorMsg;
        }

        [WebMethod]
        public ContactDeleteResult ContactDelete(Int32 nContactId)
        {
            ContactDeleteResult cResult = new ContactDeleteResult();
            try
            {
                GuptaCom.MyCom cGc = new GuptaCom.MyCom();
                cResult.bRet = cGc.ContactDelete(nContactId, ref cResult.sErrorMsg);
            }
            catch (Exception ex)
            {
                cResult.bRet = false;
                cResult.sErrorMsg = ex.Message;
            }
            return cResult;
        }

        System.Drawing.Image SizeToBestFit(System.Drawing.Image imgIn)
        {
            if (imgIn.Width > 0 && imgIn.Height > 0)
            {
                Bitmap bmp = new Bitmap(200, 300);
                Graphics gr = Graphics.FromImage((System.Drawing.Image)bmp);
                gr.FillRectangle(Brushes.White, 0, 0, 200, 300);
                gr.DrawRectangle(new Pen(Color.LightGray), 0, 0, 200, 300);
                gr.InterpolationMode = InterpolationMode.HighQualityBicubic;
                float RatioWidth = (float)200 / imgIn.Width;
                float RatioHeight = (float)300 / imgIn.Height;
                float nOffsetX = 0;
                float nOffsetY = 0;
                if (RatioWidth > RatioHeight)
                {
                    RatioWidth = RatioHeight;
                    nOffsetX = ((float)200 - imgIn.Width * RatioWidth) / 2;
                }
                else
                {
                    RatioHeight = RatioWidth;
                    nOffsetY = ((float)300 - imgIn.Height * RatioHeight) / 2;
                }
                gr.DrawImage(imgIn, nOffsetX, nOffsetY, imgIn.Width * RatioWidth, imgIn.Height * RatioHeight);
                gr.Dispose();
                return (System.Drawing.Image)bmp;
            }
            else
            {
                return imgIn;
            }
        }
    }
}
